# coding: utf-8

import re
import os
from pathlib import Path
import subprocess


class MDADMHandler:

    @staticmethod
    def check_detail(dev: str | Path) -> tuple[bool, str]:
        dev = str(dev)

        try:
            result = subprocess.run(
                ["mdadm", "--detail", dev],
                capture_output=True,
                text=True,
                check=True
            )
            output = result.stdout

            """
            Основные возможные состояния (State):
                clean – OK.
                active – OK.
                degraded – work, but there are failed disks.
                resyncing – sync.
                recovering – recover.
                reshape – change raid, or size, or number of disks.
                checking – check raid consistency.
                faulty – fail.
                inactive – stopped.
            """
            if "State :" in output and "Failed Devices :" in output:
                state_line = [line for line in output.split('\n') if "State :" in line]
                fail_line = [line for line in output.split('\n') if "Failed Devices :" in line]
                if len(state_line) != 1 or len(fail_line) != 1:
                    return False, f"Cannot understand state of \"{dev}\""
                state_line, fail_line = state_line[0], fail_line[0]
                state = state_line.split(":")[1].strip().lower()
                fails_num = int(fail_line.split(":")[1].strip())
                res_text, all_ok = "", True
                if "faulty" in state or "inactive" in state or "degraded" in state:
                    res_text += f"Problem with RAID {dev}! State: {state}. \n"
                    all_ok = False
                if fails_num > 0:
                    res_text += f"Problem with RAID {dev}! Failed Devices: {fails_num}. \n"
                if all_ok:
                    return True, ""
                else:
                    return False, res_text
            else:
                return False, f"Cannot understand state of \"{dev}\""

        except subprocess.CalledProcessError as e:
            print(f"Error executing mdadm command: {e.stderr}")
            return False, "Error "
        except Exception as e:
            return False, f"Error: {e}"

    @staticmethod
    def get_full_report(dev: str | Path) -> str:
        dev = str(dev)
        res = ""
        output = None
        try:
            result = subprocess.run(
                ["mdadm", "--detail", dev],
                capture_output=True,
                text=True,
                check=True
            )
            output = result.stdout
        except Exception as e:
            res += f"Cannot run command \"mdadm --detail {dev}\": {e} \n"

        res += f"mdadm --detail {dev}: \n\n{output} \n"

        mdstat_content = None
        try:
            mdstat_content = open("/proc/mdstat", 'r').read()
        except Exception as e:
            res += f"Cannot read file /proc/mdstat: {e} \n"

        res += f"/proc/mdstat: \n\n{mdstat_content}"

        return res


if __name__ == "__main__":
    devdisk = "/dev/md0"
    o = MDADMHandler.get_full_report(devdisk)
    print(o, "\n\n\n")
    print(MDADMHandler.check_detail(devdisk))
    """
for i in {1..10}; do
    if [ ! -f "disk$i.img" ]; then
        dd if=/dev/zero of="disk$i.img" bs=1M count=1024
    fi
done

for i in {1..10}; do
    losetup -f "disk$i.img"
done

LOOPS=$(losetup -l | grep disk | awk '{print $1}')

mdadm --create --verbose /dev/md0 --level=6 --raid-devices=10 $LOOPS

cat /proc/mdstat
mdadm --detail /dev/md0
    """



# coding: utf-8

import platform
import sys
from alerk_pack.communicator import Kommunicator

from smalk_disk_check.install_checking import install_check_and_root_check
from smalk_disk_check.args_parsing import get_args
from smalk_disk_check.setting_manager import SettingManager
from smalk_disk_check.disk import DiskManager
from smalk_disk_check.key_manager import KeyManager
from smalk_disk_check.process import main_loop


def main():
    if platform.system().lower() != "linux":
        print("smalk_disk_check can run only on GNU/Linux.")
        sys.exit(1)

    args = get_args()
    sm = SettingManager(args.settings_path)

    install_check_and_root_check()

    disk_manager = DiskManager(sm=sm)
    km = KeyManager(sm=sm)
    kommunicator = Kommunicator(url=sm.get_url(),
                                priv_key=km.get_priv_key(), public_key=km.get_pub_key(),
                                sign_key=km.get_sign_key(), verify_key=km.get_verify_key(),
                                alerk_pub_key=km.get_alerk_pub_key(), alerk_verify_key=km.get_alerk_verify_key(),
                                sym_key=km.get_sym_key())
    kommunicator.start()
    main_loop()


if __name__ == "__main__":
    main()



# coding: utf-8

import time
import traceback
import datetime
from alerk_pack.message import MessageWrapper
from alerk_pack.communicator import Kommunicator

from smalk_disk_check.setting_manager import SettingManager
from smalk_disk_check.disk import DiskManager, Disk


def check_first_time():
    sm = SettingManager()
    prefix_message = sm.get_prefix_message()

    if sm.notify_if_turned_on():
        startup_message = sm.get_startup_message()
        mw = MessageWrapper(msg_type=MessageWrapper.MSG_TYPE_REPORT, text=f"{prefix_message} {startup_message}", is_attachments=False)
        kommunicator = Kommunicator()
        kommunicator.add_msg(mw)


def main_loop():
    while True:
        try:
            sm = SettingManager()
            kommunicator = Kommunicator()
            disk_manager = DiskManager()
            disks: list[Disk] = disk_manager.get_disks()
            last_disk_check_time, last_full_report_time = None, None
            disk_polling_rate, full_report_period = sm.disk_polling_rate(), sm.full_report_period()
            prefix_message = sm.get_prefix_message()
            problem_message = sm.get_problem_message()
            full_report_message = sm.get_full_report_message()

            test_start(disks)

            check_first_time()

            while True:
                time.sleep(30)
                cur_time = time.time()
                if last_disk_check_time is None or cur_time - last_disk_check_time >= disk_polling_rate:
                    problems, text, problem_disks = disk_polls(disks)
                    if problems:
                        mw = MessageWrapper(msg_type=MessageWrapper.MSG_TYPE_REPORT, text=f"{prefix_message} {problem_message}: {problem_disks}. ",
                                            is_attachments=True)
                        raws = [("re.txt", text.encode(encoding="utf-8"))]
                        kommunicator.add_msg(mw, raws=raws)
                    last_disk_check_time = time.time()
                if last_full_report_time is None or cur_time - last_full_report_time >= full_report_period:
                    full_report = full_disk_report(disks)
                    mw = MessageWrapper(msg_type=MessageWrapper.MSG_TYPE_REPORT, text=f"{prefix_message} {full_report_message}. ",
                                            is_attachments=True)
                    raws = [("r.txt", full_report.encode(encoding="utf-8"))]
                    kommunicator.add_msg(mw, raws=raws)
                    last_full_report_time = time.time()
        except Exception as e:
            print(f"Something gone wrong: {e}. Restarting...")
            error_text = f"{traceback.format_exc()}\n{e}"
            print(error_text)
            time.sleep(5)


def get_cur_time():
    template = "%Y.%m.%d %H:%M:%S"
    time_str = datetime.datetime.now().strftime(template)
    return time_str


def test_start(disks: list[Disk]):
    try:
        if_problem, problem_text, problem_disk_code_list = disk_polls(disks)
        if if_problem:
            print("Cannot test check disks. Is all settings correct? ")
            print(f"Problems disks: {problem_disk_code_list}, text: \n{problem_text}")
            if SettingManager().get_interactive():
                user_input_orig = input("Press Enter to exit or type \"continue\" to continue: ")
                user_input = user_input_orig.strip().lower()
                if user_input == "":
                    exit(1)
                elif user_input == "continue":
                    return
                else:
                    print(f"Cannot understand \"{user_input_orig}\". Exit.")
                    exit(1)
            else:
                return

        full_disk_report(disks)
    except Exception as e:
        error_text = f"{traceback.format_exc()}\n{e}"
        print(error_text)
        print("Cannot test check disks. Is all settings correct? ")
        exit(1)


def disk_polls(disks: list[Disk]) -> tuple[bool, str, list[str]]:
    """

    :param disks:
    :return: if_problem, problem_text, problem_disk_code_list
    """
    cur_time = get_cur_time()
    delimiter = "="*80
    if_problem, problem_text, problem_disk_code_list = False, "", []
    for disk_i in disks:
        try:
            check_success, text = disk_i.check()
            if not check_success:
                problem_text += f"{delimiter}\nDisk \"{disk_i.get_name()}\" ({disk_i.get_dev_path()}) has problem: \n{text}\n\n\n"
                if_problem = True
                problem_disk_code_list.append(disk_i.get_code())
        except Exception as e:
            error_text = f"{traceback.format_exc()}\n{e}"
            problem_text += f"{delimiter}\nDisk \"{disk_i.get_name()}\" ({disk_i.get_dev_path()}) error while checking: \n{error_text}\n\n\n"
            if_problem = True
            problem_disk_code_list.append(disk_i.get_code())
    if if_problem:
        problem_text = f"{cur_time}: \n\n\n" + problem_text
    return if_problem, problem_text, problem_disk_code_list,


def full_disk_report(disks: list[Disk]) -> str:
    cur_time = get_cur_time()
    res = f"{cur_time}: \n\n\n"
    delimiter = "="*80
    for disk_i in disks:
        try:
            disk_i_report_text = disk_i.get_report()
            res += f"{delimiter}\nDisk \"{disk_i.get_name()}\" ({disk_i.get_dev_path()}): \n\n\n{disk_i_report_text}"
        except Exception as e:
            error_text = f"{traceback.format_exc()}\n{e}"
            res += f"{delimiter}\nDisk \"{disk_i.get_name()}\" ({disk_i.get_dev_path()}) error while checking: \n{error_text}\n\n\n"

    return res



# coding: utf-8

import subprocess
import os

from smalk_disk_check.setting_manager import SettingManager


def sys_check_program(program):
    # shutil.which ???
    try:
        subprocess.run([program, '--version'], stdout=subprocess.PIPE, stderr=subprocess.PIPE, check=True)
        return True
    except subprocess.CalledProcessError:
        print(f"WARNING: command \"{program} --version\" exited with error, but \"{program}\" installed. ")
        return True
    except FileNotFoundError:
        return False


def root_check() -> bool:
    if os.geteuid() == 0:
        return True
    else:
        return False


def install_check_and_root_check():
    if not root_check():
        print("This program must be run as root. ")
        exit()
    programs = {
        "lsblk": "util-linux",
        "mdadm": "mdadm",
        "smartctl": "smartmontools",
        "hddtemp": "hddtemp",
        "hdparm": "hdparm"
        # "sensors": "lm-sensors",
    }

    f = False
    for program, package in programs.items():
        if not sys_check_program(program):
            if program != "hddtemp":
                f = True
                print(f"\"{program}\" is not installed. Try to install it with package: \"{package}\". ")
            else:
                print(f"\"{program}\" is not installed. Try to install it with package: \"{package}\". smalk_disk_check can run without hddtemp, but you must check it first with tests. ")
                if SettingManager().get_interactive():
                    input("Press Enter to continue")

    if f:
        print("Be sure, what all of these utils installed on your system: lsblk, mdadm, smartctl, hddtemp -- and rerun. ")
        print("The problem may also be related to the fact that these programs are not installed for the root user, but are installed for regular users.")
        exit()



# coding: utf-8

import argparse
from smalk_disk_check import __version__


def get_args() -> argparse.Namespace:
    parser = argparse.ArgumentParser(description="smalk_disk_check is smalk implementation for disk checking and monitoring. ")

    parser.add_argument("--version", action="version", version=f"V{__version__}", help="Check version. ")

    parser.add_argument("settings_path", type=str, help="Path to yaml file with settings. ")

    return parser.parse_args()



# coding: utf-8

from pathlib import Path
import subprocess
from pySMART import Device

class SMARTHandler:

    @staticmethod
    def get_smart_table(dev: str | Path, dev_type: str | None) -> dict[int: int]:
        dev = str(dev)
        if dev_type is None:
            smart = Device(dev)
        else:
            smart = Device(dev, interface=dev_type)

        smart_attributes = smart.attributes

        smart_dict: dict[int: int] = {}

        for attribute in smart_attributes:
            if attribute is not None:
                # smart_dict[attribute.num] = str(attribute.raw)
                smart_dict[attribute.num] = attribute.raw_int

        return smart_dict

    @staticmethod
    def try_get_temperature(dev: str | Path, dev_type: str | None) -> int | None:
        dev = str(dev)
        try:
            if dev_type is not None:
                smart = Device(dev)
            else:
                smart = Device(dev, interface=dev_type)

            res = smart.temperature
            return res
        except Exception as e:
            return None

    @staticmethod
    def get_smart_a_of(dev: str | Path, dev_type: str | None) -> str:
        dev = str(dev)
        if dev_type is not None:
            cmd = ["smartctl", "-a", "-d", dev_type, dev]
        else:
            cmd = ["smartctl", "-a", dev]
        try:
            result = subprocess.run(cmd, capture_output=True, text=True, check=True)
            return result.stdout
        except subprocess.CalledProcessError as e:
            raise ValueError(f"Cannot run cmd {cmd}: {e.stderr}")

    @staticmethod
    def get_smart_x_of(dev: str | Path, dev_type: str | None) -> str:
        dev = str(dev)
        if dev_type is not None:
            cmd = ["smartctl", "-x", "-d", dev_type, dev]
        else:
            cmd = ["smartctl", "-x", dev]
        try:
            result = subprocess.run(cmd, capture_output=True, text=True, check=True)
            return result.stdout
        except subprocess.CalledProcessError as e:
            raise ValueError(f"Cannot run cmd {cmd}: {e.stderr}")



# coding: utf-8

import re
from pathlib import Path
import subprocess
import time


class TempHandler:

    @staticmethod
    def get_temp(dev: str | Path) -> int | None:
        dev = str(dev)

        def run_hddtemp() -> str | None:
            try:
                result_l = subprocess.run(
                    ["hddtemp", dev],
                    capture_output=True,
                    text=True,
                    check=True
                )
                # print(result_l.stdout, " ::: ", result_l.stderr)
                return result_l.stdout.strip()
            except subprocess.CalledProcessError:
                return None
            except Exception as e:
                return None

        def wake_disk() -> bool:
            try:
                subprocess.run(["hdparm", "-S0", dev], capture_output=True, text=True)
                subprocess.run(["hdparm", "-I", dev], capture_output=True, text=True)
                return True
            except Exception as e:
                print(f"Error while wake up with hdparm: {e}")
                return False

        try:
            output = run_hddtemp()

            if "is sleeping" in output.lower():  # sudo hdparm -Y /dev/sda
                # print(f"Device {dev} is sleeping. Trying to wake up... ")
                if wake_disk():
                    time.sleep(5)
                    output = run_hddtemp()
                    if not output:
                        return None
                else:
                    return None

            try:
                return int(output[output.rfind(": ")+2: output.rfind("°C")])  # : 33°C
            except Exception as e:
                return None
            # match = re.search(r'(\d+)\s*°?', output)
            # if match:
            #     try:
            #         return int(match.group(1))
            #     except ValueError:
            #         return None

        except Exception as e:
            print(f"Unexpected error: {e}")
            return None


if __name__ == "__main__":
    t = TempHandler.get_temp("/dev/sda")
    print(t)



# coding: utf-8

from ksupk import singleton_decorator
from alerk_pack.crypto import str_to_asym_key, str_to_sym_key, AES256Key, RSAPrivateKey, RSAPublicKey

from smalk_disk_check.setting_manager import SettingManager


@singleton_decorator
class KeyManager:

    def __init__(self, sm: SettingManager):
        self.priv_key, self.pub_key, self.sign_key, self.verify_key = sm.get_my_keys()
        self.alerk_pub_key, self.alerk_verify_key = sm.get_alerk_keys()
        self.sym_key_str = sm.get_sym_key()

        self.priv_key: RSAPrivateKey = str_to_asym_key(self.priv_key, False)
        self.pub_key: RSAPublicKey = str_to_asym_key(self.pub_key, True)
        self.sign_key: RSAPrivateKey = str_to_asym_key(self.sign_key, False)
        self.verify_key: RSAPublicKey = str_to_asym_key(self.verify_key, True)

        self.alerk_pub_key: RSAPublicKey = str_to_asym_key(self.alerk_pub_key, True)
        self.alerk_verify_key: RSAPublicKey = str_to_asym_key(self.alerk_verify_key, True)

        self.sym_key: AES256Key | None = None
        if self.sym_key_str.strip() == "":
            self.sym_key = None
        else:
            self.sym_key = str_to_sym_key(self.sym_key_str)

    def get_priv_key(self) -> RSAPrivateKey:
        return self.priv_key

    def get_pub_key(self) -> RSAPublicKey:
        return self.pub_key

    def get_sign_key(self) -> RSAPrivateKey:
        return self.sign_key

    def get_verify_key(self) -> RSAPublicKey:
        return self.verify_key

    def get_alerk_pub_key(self) -> RSAPublicKey:
        return self.alerk_pub_key

    def get_alerk_verify_key(self) -> RSAPublicKey:
        return self.alerk_verify_key

    def get_sym_key(self) -> AES256Key | None:
        return self.sym_key



# coding: utf-8

import re
from ksupk import is_int
from typing import Callable
import subprocess
import traceback
from pathlib import Path
from ksupk import singleton_decorator

from smalk_disk_check.setting_manager import SettingManager
from smalk_disk_check.smart_handler import SMARTHandler
from smalk_disk_check.mdadm_handler import MDADMHandler
from smalk_disk_check.temp_handler import TempHandler


def is_valid_attribute_check_condition(condition: str) -> bool:
    pattern1 = r"^x\s*(>=|<=|==|>|<|=|!=)\s*-?\d+$"
    pattern2 = r"^-?\d+\s*(>=|<=|==|>|<|=|!=)\s*x$"
    return (re.match(pattern1, condition.replace(" ", "")) is not None or
            re.match(pattern2, condition.replace(" ", "")) is not None)


def get_lsblk_info() -> list[dict[str: str]]:
    try:
        result = subprocess.run(['lsblk', '-o', 'NAME,UUID,LABEL'], capture_output=True, text=True)
        lines = result.stdout.strip().split('\n')
    except subprocess.CalledProcessError as e:
        raise RuntimeError(f"Error of running \"lsblk -o NAME,UUID\": {e}")

    def remove_non_english_characters(input_string) -> str:
        return re.sub(r'[^a-zA-Z]', '', input_string)

    lsblk_list = []
    for line in lines[1:]:
        parts = line.split()
        name = remove_non_english_characters(parts[0])
        lsblk_list.append({
            "NAME": name,
            "UUID": parts[1] if len(parts) > 1 else None,
            # 'LABEL': parts[2] if len(parts) > 2 else None
        })

    return lsblk_list


class Disk:
    def __init__(self, name: str, code: str, dev_path: Path, disk_type: str, max_temp: int | None,
                 smart_attr_to_check: dict[int | str: Callable[[int], bool] | str] | None):
        self.name: str = name
        self.code: str = code
        self.dev_path: Path = dev_path

        disk_type = disk_type.strip().lower()
        if disk_type == "mdadm":
            self.mdadm: bool = True
        else:
            self.mdadm: bool = False
        self.disk_type: str = disk_type

        self.max_temp: int | None = max_temp
        self.smart_attr_to_check = smart_attr_to_check

    def check(self) -> tuple[bool, str]:
        """

        :return: check_success, problem_text
        """
        if not self.check_if_in_system():
            return False, f"No such file: \"{self.get_dev_path()}\". "
        if not self.try_read():
            return False, f"Cannot read device \"{self.get_dev_path()}\". "
        problem, res = False, ""
        if self.get_max_temp() is not None and not self.is_mdadm():
            t = self.get_temp()
            if t is None:
                problem = True
                res += f"Cannot get temperature of device \"{self.get_dev_path()}\". \n"
            if t is not None and t > self.get_max_temp():
                problem = True
                res += f"Temperature of disk is too high: t={t}°C > max={self.get_max_temp()}°C. \n"
        if not self.is_mdadm():
            smart_table = self.get_smart_table()
            if smart_table is None:
                problem = True
                res += f"Cannot get S.M.A.R.T. from device \"{self.get_dev_path()}\". \n"
            try:
                f, o = self.check_smart_attributes(smart_table)
                if not f:
                    problem = True
                    res += f"Problem with S.M.A.R.T.: \n{o} \n"
                    res += f"smartctl -a {self.get_dev_path()}: \n {SMARTHandler.get_smart_a_of(self.get_dev_path(), self.get_disk_type())} \n"
            except Exception as e:
                problem = True
                res += f"Error while checking S.M.A.R.T.: {e}. \n"
        else:
            mdadm_check_f, mdadm_check_m = MDADMHandler.check_detail(self.get_dev_path())
            if not mdadm_check_f:
                problem = True
                res += f"Problem with RAID: \n{mdadm_check_m} \n"
                res += f"mdadm full report: \n{MDADMHandler.get_full_report(self.get_dev_path())} \n"

        if problem:
            res = f"{str(self)} \n" + res

        return not problem, res

    def get_report(self) -> str:
        res = ""
        if not self.check_if_in_system():
            res += f"No such file: \"{self.get_dev_path()}\". \n"
        if not self.try_read():
            res +=  f"Cannot read device \"{self.get_dev_path()}\". \n"

        if self.get_max_temp() is not None and not self.is_mdadm():
            t = self.get_temp()
            if t is None:
                res += f"Cannot get temperature of device \"{self.get_dev_path()}\". \n"
            else:
                res += f"Temperature of {self.get_dev_path()} is {t}°C (max={self.get_max_temp()}°C). \n"
            if t is not None and t > self.get_max_temp():
                res += f"Temperature of disk is too high: t={t}°C > max={self.get_max_temp()}°C. \n"

        if not self.is_mdadm():
            res += f"\n\nsmartctl -a {self.get_dev_path()}: \n {SMARTHandler.get_smart_a_of(self.get_dev_path(), self.get_disk_type())} \n"
            buffs, _smart_table = "", self.get_smart_table()
            if _smart_table is not None:
                for attr, val in _smart_table.items():
                    buffs += f"{attr} --- {val}\n"
            else:
                buffs = f"{_smart_table}"
            res += f"smart table: \n{buffs} \n"
        else:
            res += f"\n\nmdadm full report: \n{MDADMHandler.get_full_report(self.get_dev_path())} \n"

        return res

    def get_name(self) -> str:
        return self.name

    def get_code(self) -> str:
        return self.code

    def get_temp(self) -> int | None:
        tmp1 = TempHandler.get_temp(self.get_dev_path())
        if tmp1 is None:
            tmp2 = SMARTHandler.try_get_temperature(self.get_dev_path(), self.get_disk_type())
            # assert tmp1 == tmp2
            return tmp2
        else:
            return tmp1

    def try_read(self) -> bool:
        try:
            with open(self.get_dev_path(), 'rb') as f:
                f.read(1024)
            return True
        except Exception as e:
            return False

    def check_if_in_system(self) -> bool:
        if Path(self.get_dev_path()).exists():
            return self.try_read()
        else:
            return False

    def get_smart_table(self) -> dict[int: int] | None:
        try:
            smart_table: dict[int: int] = SMARTHandler.get_smart_table(self.get_dev_path(), self.get_disk_type())
            if len(smart_table) == 0:
                raise RuntimeError(f"(Disk.get_smart_table) smart table has len zero. S.M.A.R.T. is \"{smart_table}\"")
            else:
                return smart_table
        except Exception as e:
            error_text = f"{traceback.format_exc()}\n{e}"
            print(error_text)
            return None

    def check_smart_attributes(self, smart_table: dict[int: int]) -> tuple[bool, str]:
        # try:
        #     smart_table: dict[int: int] = SMARTHandler.get_smart_table(self.get_dev_path(), self.get_disk_type())
        # except Exception as e:
        #     return False, "Cannot get S.M.A.R.T."

        res_str = ""
        for attr_num in self.smart_attr_to_check:
            if isinstance(attr_num, int):
                pass
            elif isinstance(attr_num, str):
                continue
            else:
                raise ValueError(f"(Disk.check_smart_attributes) attr_num \"{attr_num}\" must be only int or str, not {type(attr_num)}")
            if attr_num not in smart_table:
                raise RuntimeError(f"Cannot find attribute {attr_num} in S.M.A.R.T. ")
            value = smart_table[attr_num]
            lambda_f = self.smart_attr_to_check[attr_num]
            lambda_f_str = self.smart_attr_to_check[str(attr_num)]
            if lambda_f(value):
                res_str += f"S.M.A.R.T. is not passed. Attribute {attr_num} check fail. Attribute {attr_num} is \"{value}\", but problem if: \"{lambda_f_str}\". \n"

        if res_str == "":
            return True, res_str
        else:
            return False, res_str

    def get_max_temp(self) -> int | None:
        return self.max_temp

    def get_dev_path(self) -> Path:
        return Path(self.dev_path)

    def get_disk_type(self) -> str:
        return self.disk_type

    def is_mdadm(self) -> bool:
        return self.mdadm

    def __str__(self):
        smart_attr_to_check = {}
        if not self.is_mdadm():
            for k_i in self.smart_attr_to_check:
                if isinstance(k_i, str):
                    smart_attr_to_check[k_i] = smart_attr_to_check[k_i]
                else:
                    continue
        return (f"Disk(name={self.name}, code={self.code}, dev_path={self.dev_path}), disk_type={self.disk_type}, is_mdadm={self.mdadm}, "
                f"max_temp={self.max_temp}, smart_attr_to_check={smart_attr_to_check})")


@singleton_decorator
class DiskManager:

    def __init__(self, sm: SettingManager):
        disks = sm.get_disks()
        self.disks: list[Disk] = []

        dev_paths = []
        for disk_i in disks:
            self._check_corrent_of(disk_i)
            disk_name = disk_i["name"]
            disk_code = disk_i["code"]
            if disk_i["define_type"] == "dev":
                dev_path = self._check_define_dev(disk_i["disk"], disk_name)
            elif disk_i["define_type"] == "by-id":
                dev_path = self._check_define_by_id(disk_i["disk"], disk_name)
            elif disk_i["define_type"] == "uuid":
                dev_path = self._find_disk_by_uuid(disk_i["disk"], disk_name)
            else:
                raise ValueError(f"(DiskManager.__init__) Failed successfully.")
            dev_paths.append(dev_path)

            disk_type = disk_i["type"]

            if is_int(disk_i["max_temp"]):
                max_temp = int(disk_i["max_temp"])
            elif str(disk_i["max_temp"]).strip().lower() == "none":
                max_temp = None
            else:
                raise ValueError(f"Cannot understand \"max_temp\"=\"{disk_i['max_temp']}\" of disk {disk_name}")

            if disk_type != "mdadm":
                smarts = disk_i["smart_check"]
                smart_attr_to_check: dict[int | str: Callable[[int], bool] | str] = {}
                for smart_i in smarts:
                    attr_num: int = int(smart_i["attribute_num"])
                    condition: str = smart_i["problem_if"]
                    smart_attr_to_check[attr_num] = eval(f"lambda x: {condition}")
                    smart_attr_to_check[str(attr_num)] = condition
            else:
                smart_attr_to_check = None
            disk = Disk(name=disk_name, code=disk_code, dev_path=dev_path, disk_type=disk_type, max_temp=max_temp,
                        smart_attr_to_check=smart_attr_to_check)
            self.disks.append(disk)

        for string in dev_paths:
            if dev_paths.count(string) > 1:
                raise ValueError(f"Several repetitions of the device \"{string}\"")

    def _check_corrent_of(self, disk_record: dict):
        if "name" not in disk_record:
            raise ValueError(f"\"name\" not defined for disk. ")
        name = disk_record["name"]
        if "code" not in disk_record:
            raise ValueError(f"\"code\" not defined for disk. Make it the same as the \"name\" if you do not know why it is needed. ")
        code = disk_record["code"]
        if "define_type" not in disk_record:
            raise ValueError(f"\"define_type\" not defined for disk \"{name}\" ({code}). ")
        allowed_define_types = ["dev", "by-id", "uuid"]
        if disk_record["define_type"] not in allowed_define_types:
            raise ValueError(f"\"define_type\" must be only: {allowed_define_types}, not \"{disk_record['define_type']}\"")
        if "disk" not in disk_record:
            raise ValueError(f"\"disk\" not defined for disk \"{name}\" ({code}). ")
        if "type" not in disk_record:
            raise ValueError(f"\"type\" not defined for disk \"{name}\" ({code}). ")
        if "max_temp" not in disk_record:
            raise ValueError(f"\"max_temp\" not defined for disk \"{name}\" ({code}). ")
        if disk_record["type"] != "mdadm" and "smart_check" not in disk_record:
            raise ValueError(f"\"smart_check\" not defined for disk \"{name}\" ({code}). ")
        if disk_record["type"] != "mdadm":
            smarts = disk_record["smart_check"]
            for smart_i in smarts:
                if "attribute_num" not in smart_i:
                    raise ValueError(f"\"attribute_num\" not defined: {smart_i}. Disk \"{name}\" ({code}). ")
                if not isinstance(smart_i["attribute_num"], int):
                    raise ValueError(f"\"attribute_num\" must be int, not {type(smart_i['attribute_num'])}. Disk \"{name}\" ({code}). ")
                if "problem_if" not in smart_i:
                    raise ValueError(f"\"problem_if\" not defined: {smart_i}. Disk \"{name}\" ({code}). ")
                if not isinstance(smart_i["problem_if"], str):
                    raise ValueError(f"\"problem_if\" must be int, not {type(smart_i['problem_if'])}. Disk \"{name}\" ({code}). ")
                if not is_valid_attribute_check_condition(smart_i["problem_if"]):
                    raise ValueError(f"\"problem_if\" contains wrong condition: \"{smart_i['problem_if']}\". Disk \"{name}\" ({code}). ")

    def _check_define_dev(self, path: str, disk_name: str) -> Path:
        # pattern = r'^/dev/sd[a-z]$'  # nvme? mdX? scsi?
        pattern = r'^/dev/[a-zA-Z0-9]+$'
        if not re.match(pattern, path):
            raise ValueError(f"Cannot understand this: \"{path}\" -- of disk {disk_name}. ")
        res = Path(path)
        if res.exists():
            return res
        else:
            raise ValueError(f"File \"{str(res)}\" does not exist. ")

    def _check_define_by_id(self, path: str, disk_name: str) -> Path:
        pattern = r'^/dev/disk/by-id/[a-zA-Z0-9_-]+$'
        if not re.match(pattern, path):
            raise ValueError(f"Cannot understand this: \"{path}\" -- of disk {disk_name}. ")

        res = Path(path)
        if res.exists():
            if res.is_symlink():
                return res.resolve()
            else:
                return res
        else:
            raise ValueError(f"File \"{str(res)}\" does not exist. ")

    def _find_disk_by_uuid(self, uuid: str, disk_name: str) -> Path:
        uuid = uuid.strip().lower()
        lsblk: list[dict[str: str]] = get_lsblk_info()

        disks = []
        for record in lsblk:
            name, disk_uuid = str(record["NAME"]), str(record["UUID"])
            if disk_uuid.strip().lower() == uuid:
                disks.append(f"/dev/{name}")

        if len(disks) == 0:
            raise ValueError(f"Cannot find disk, where volume \"{uuid}\" exists. ")
        elif len(disks) > 1:
            raise ValueError(f"Multiple disks with the specified UUID (\"{uuid}\") were found. Disk \"{disk_name}\"")
        else:
            res = Path(disks[0])

        return res

    def get_disks(self) -> list[Disk]:
        return self.disks



# coding: utf-8

import yaml
from pathlib import Path
from ksupk import singleton_decorator


@singleton_decorator
class SettingManager:

    def __init__(self, settings_yaml_path: Path | str):
        settings_yaml_path = Path(settings_yaml_path)

        with open(settings_yaml_path, 'r', encoding="utf-8") as file:
            data = yaml.safe_load(file)

        self.data: dict = dict(data)

    def get_disks(self) -> list:
        return self.data["disk"]

    def get_my_keys(self) -> tuple[str, str, str, str]:
        return self.data["keys"]["priv_key"], self.data["keys"]["pub_key"], self.data["keys"]["sign_key"], self.data["keys"]["verify_key"]

    def get_alerk_keys(self) -> tuple[str, str]:
        return self.data["alerk"]["pub_key"], self.data["alerk"]["verify_key"]

    def get_sym_key(self) -> str:
        return self.data["app"]["cipher_key"]

    def get_alerk_conn(self) -> tuple[str, int, str]:
        return self.data["alerk"]["ip"], self.data["alerk"]["port"], self.data["alerk"]["endpoint"]

    def get_protocol(self) -> str:
        res: str = self.data["alerk"]["protocol"]
        if res.strip().lower() not in ["http", "https"]:
            raise ValueError("protocol must be only: \"http\" or \"https\"")
        return res

    def get_url(self) -> str:
        protocol = self.get_protocol().strip().lower()
        ip, port, endpoint = self.get_alerk_conn()
        return f"{protocol}://{ip}:{port}{endpoint}"

    def notify_if_turned_on(self) -> bool:
        return self.data["app"]["notify_if_turned_on"]

    def disk_polling_rate(self) -> int:
        return self.data["app"]["disk_polling_rate"]

    def full_report_period(self) -> int:
        return self.data["app"]["full_report_period"]

    def get_prefix_message(self) -> str:
        return self.data["app"]["prefix_message"]

    def get_startup_message(self) -> str:
        return self.data["app"]["startup_message"]

    def get_problem_message(self) -> str:
        return self.data["app"]["problem_message"]

    def get_full_report_message(self) -> str:
        return self.data["app"]["full_report_message"]

    def get_interactive(self) -> bool:
        return self.data["app"]["interactive"]
